#include once "rendering.bi"

const as f32 MASS = 100
function CreateCombinedRigidBody(sim as neSimulator ptr) as neRigidBody ptr
  var rb = SimulatorCreateRigidBody(sim)
  var g0 = RigidBodyAddGeometry(rb)
  var g1 = RigidBodyAddGeometry(rb)
  var g2 = RigidBodyAddGeometry(rb)
  var g3 = RigidBodyAddGeometry(rb)
  var g4 = RigidBodyAddGeometry(rb)
  
  var p0 = neV3(0,0,0)
  var p1 = neV3(-1,0,-1)
  var p2 = neV3(-1,0, 1)
  var p3 = neV3( 1,0, 1)
  var p4 = neV3( 1,0,-1)
  var zero = neV3()
  GeometrySetSphereDiameter(g0,1.5) : GeometrySetPositionRotationV3(g0,p0,zero)
  GeometrySetBoxSize(g1,1,1,1) : GeometrySetPositionRotationV3(g1,p1, zero)
  GeometrySetBoxSize(g2,1,1,1) : GeometrySetPositionRotationV3(g2,p2, zero)
  GeometrySetBoxSize(g3,1,1,1) : GeometrySetPositionRotationV3(g3,p3, zero)
  GeometrySetBoxSize(g4,1,1,1) : GeometrySetPositionRotationV3(g4,p4, zero)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function  
'
' main
'
var sim = CreateSimulator()

var ground = CreateAnimatedBodyBox(sim,500,2,500)
AnimatedBodySetPosition(ground,0,-1,0) ' make top face at zero (ground)

var rb1 = CreateCombinedRigidBody(sim)
RigidBodySetInertiaTensor(rb1,neSphereInertiaTensor(3, MASS*5))
RigidBodySetMass(rb1,MASS*5)
RigidBodyUpdateBoundingInfo(rb1)
RigidBodySetPositionRotation(rb1,-3,1.75,0, 120,0,0)

var rb2 = CreateCombinedRigidBody(sim)
RigidBodySetInertiaTensor(rb2,neBoxInertiaTensor(3,1.5,3,MASS*5))
RigidBodySetMass(rb2,MASS*5)
RigidBodyUpdateBoundingInfo(rb2)
RigidBodySetPositionRotation(rb2, 3,1.75,0, 120,0,0)

dim as RENDERER gfx
dim as integer frame
while inkey()=""
  SimulatorAdvance(sim) ' 60 Hz.
  if frame mod 2=0 then ' 30 Hz.
    gfx.cls
    glLoadIdentity()
    glTransLatef(0,-4,-20) 
    gfx.Render(ground)
    gfx.Render(rb1)
    gfx.Render(rb2)
    gfx.Swapbuffers()
  else
    sleep(10) ' don't use all CPU cycles
  end if
  frame+=1
wend
DestroySimulator(sim)


