#ifndef __redering_bi__
#define __redering_bi__

#include once "tokamak-c.bi"
#include once "GL/gl.bi"
#include once "fbgfx.bi"

#if __FB_LANG__ = "fb"
using FB
#endif

#ifndef M_PI
const as single M_PI = atn(1)*4
#endif

' same as gluPerspective
sub Perspective(byval fov as double,byval ratio as double,byval zNear as double,byval zFar as double)
  dim as double fH = tan( fov / 360.0 * 3.1415926535897932384626433832795 ) * zNear
  dim as double fW = fH * ratio
  glFrustum(-fW,fW,-fH,fH,zNear,zFar)
end sub
' same as gluLockAt
sub LookAt(byval ex as double,byval ey as double,byval ez as double, _
           byval lx as double,byval ly as double,byval lz as double, _
           byval ux as double,byval uy as double,byval uz as double )
  dim as double z(2) = {ex-lx,ey-ly,ez-lz}
  dim as double l=z(0)*z(0) + z(1)*z(1) + z(2)*z(2)
  if (l) then
    l=1.0/sqr(l)
    z(0)*=l : z(1)*=l : z(2)*=l
  end if
  dim as double x(2)={uy*z(2)-uz*z(1), _
                     -ux*z(2)+uz*z(0), _
                      ux*z(1)-uy*z(0)}
  l=x(0)*x(0) + x(1)*x(1) + x(2)*x(2)
  if (l) then
    l=1.0/sqr(l)
    x(0)*=l : x(1)*=l : x(2)*=l
  end if
  dim as double y(2)={z(1)*x(2)-z(2)*x(1), _
                     -z(0)*x(2)+z(2)*x(0), _
                      z(0)*x(1)-z(1)*x(0)}

  l=y(0)*y(0) + y(1)*y(1) + y(2)*y(2)
  if (l) then
    l=1.0/sqr(l)
    y(0)*=l : y(1)*=l : y(2)*=l
  end if
  dim as single m4x4(15)=>{x(0),y(0),z(0),0, _
                           x(1),y(1),z(1),0, _
                           x(2),y(2),z(2),0, _
                             0,    0,   0,1}
  glMultMatrixf @m4x4(0)
  glTranslatef -ex,-ey,-ez
end sub


type Renderer
  public:
  declare constructor(byval w as integer=-1,byval h as integer=-1) 
  declare function getWidth  as integer
  declare function getHeight as integer
  declare sub Render(byval ab as neAnimatedBody ptr, byval r as f32,byval g as f32,byval b as f32, byval drawSensors as boolean=false)
  declare sub Render(byval rb as neRigidBody    ptr, byval r as f32,byval g as f32,byval b as f32, byval drawSensors as boolean=false)
  declare sub Render(byval ab as neAnimatedBody ptr, byval drawSensors as boolean=false)
  declare sub Render(byval rb as neRigidBody    ptr, byval drawSensors as boolean=false)
  declare sub Render(byref tr as neTriangleMesh)
  declare sub Render(byval jo as neJoint ptr)
  declare sub SwapBuffers
  declare sub Cls(byval pixels  as boolean=true, _
                  byval depth   as boolean=true, _
                  byval stencil as boolean=false)
  private:
  declare sub Render(byval g as neGeometry ptr)
  declare sub Render(byval s as neSensor ptr)
  
  declare function CreateCube   as GLuint
  declare function CreateSphere as GLuint
  declare sub      CreateCapsule(byref Top as GLuint, byref Body as GLuint, byref Bottom as GLuint)
  as integer iW,iH,iB
  as GLuint BoxList,SphereList,CapsuleTop,CapsuleBody,CapsuleBottom
  const as integer SEGMENTS = 8
  const as single  US       = (M_PI*2) / SEGMENTS
  const as single  VS       =  M_PI    / SEGMENTS
  const as integer NPOINTS  = (SEGMENTS + 1) * (SEGMENTS + 1)      
  as single P(NPOINTS*3-1)
end type

constructor Renderer(byval w as integer,byval h as integer) 
  screeninfo iW,iH,iB
  if w=-1 or h=-1 then
    w = iW*0.75 : h = iH*0.75
  else
    if w>iW then w=iW
    if h>iH then h=iH
  end if
  screenres w,h,iB,,GFX_OPENGL
  screeninfo iW,iH,iB

  glClearColor(.5,.5,.9,1.0)
  glEnable(GL_LIGHTING)
  glEnable(GL_COLOR_MATERIAL)
  glCullFace(GL_BACK) : glEnable(GL_CULL_FACE)
  glEnable(GL_AUTO_NORMAL)
  
  dim as single amb(...) => {0.2,0.2,0.2,1.0}
  dim as single dif(...) => {0.8,0.8,0.8,1.0}
  dim as single spec(...) => {1,1,1,1}
  dim as single lp(...) => {0,100,0,0}
  glEnable (GL_LIGHT0)
  glLightfv(GL_LIGHT0,GL_AMBIENT ,@amb(0))
  glLightfv(GL_LIGHT0,GL_DIFFUSE ,@dif(0))
  glLightfv(GL_LIGHT0,GL_SPECULAR,@spec(0))
  glLightfv(GL_LIGHT0,GL_POSITION,@lp(0))

  glViewport(0,0,iW,iH)
  glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
  Perspective 45.0, iW/iH, 1, 4096
  glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()  
    
  glShadeModel(GL_SMOOTH) 
	glEnable(GL_DEPTH_TEST) 
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST) 
  'glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  'glEnable(GL_BLEND)
  glClear(GL_COLOR_BUFFER_BIT) : flip
  glClear(GL_COLOR_BUFFER_BIT) : flip
  BoxList      = CreateCube()
  SphereList   = CreateSphere()
  CreateCapsule(CapsuleTop,CapsuleBody,CapsuleBottom)
end constructor
function Renderer.getWidth  as integer
  return iW
end function
function Renderer.getHeight as integer
  return iH
end function
sub Renderer.SwapBuffers
  flip
end sub
sub Renderer.cls(byval pixels  as boolean, _
                 byval depth   as boolean, _
                 byval stencil as boolean)
  glClear(iif(pixels=true ,GL_COLOR_BUFFER_BIT  ,0) or _
          iif(depth=true  ,GL_DEPTH_BUFFER_BIT  ,0) or _
          iif(stencil=true,GL_STENCIL_BUFFER_BIT,0))
end sub

'  ###############################
' # create a unit 1 x 1 x 1 box #
'###############################  
function Renderer.CreateCube() as GLuint
  var ret = glGenLists(1)
  glNewList(ret,GL_COMPILE)
  glBegin(GL_QUADS)
    glNormal3f(0,0,1)
    glVertex3f(-0.5, 0.5,0.5):glVertex3f(-0.5,-0.5,0.5)
    glVertex3f( 0.5,-0.5,0.5):glVertex3f( 0.5, 0.5,0.5)
    glNormal3f(1,0,0)
    glVertex3f( 0.5, 0.5, 0.5):glVertex3f( 0.5,-0.5, 0.5)
    glVertex3f( 0.5,-0.5,-0.5):glVertex3f( 0.5, 0.5,-0.5)
    glNormal3f(0,0,-1)
    glVertex3f( 0.5, 0.5,-0.5):glVertex3f( 0.5,-0.5,-0.5)
    glVertex3f(-0.5,-0.5,-0.5):glVertex3f(-0.5, 0.5,-0.5)
    glNormal3f(-1,0,0)
    glVertex3f(-0.5, 0.5,-0.5):glVertex3f(-0.5,-0.5,-0.5)
    glVertex3f(-0.5,-0.5, 0.5):glVertex3f(-0.5, 0.5, 0.5)
    glNormal3f(0,1,0)
    glVertex3f(-0.5, 0.5,-0.5):glVertex3f(-0.5, 0.5, 0.5)
    glVertex3f( 0.5, 0.5, 0.5):glVertex3f( 0.5, 0.5,-0.5)
    glNormal3f(0,-1,0)
    glVertex3f(-0.5,-0.5, 0.5):glVertex3f(-0.5,-0.5,-0.5)
    glVertex3f( 0.5,-0.5,-0.5):glVertex3f( 0.5,-0.5, 0.5)
  glEnd()
  glEndList()
  return ret
end function 

'  ##################################
' # create a unit 1 x 1 x 1 sphere #
'################################## 
function Renderer.CreateSphere() as GLuint
  dim as GLuint list
  dim as integer pc,yc,xc,p0,p1,p2,p3
  dim as single VW,UW,UR,YP,VR
  dim as neV3 A,B,C,D,N
  
  For yc = 0 To SEGMENTS
    UR = Sin(VW) * 0.5
    YP = Cos(VW) * 0.5
    VR = Sin(VW) * 0.5
    VW = VW+VS
    UW = 0
    For xc = 0 To SEGMENTS
      P(PC*3+0)=Sin(     UW) * UR
      P(PC*3+1)=YP
      P(PC*3+2)=Cos(m_PI+UW) * VR
      PC=PC+1: UW=UW+US
    Next
  Next

  list=glGenLists(1)
  glNewList(list, GL_COMPILE)
    glBegin(GL_TRIANGLES)
      For yc= 0 To SEGMENTS - 1
        For xc= 0 To SEGMENTS - 1
          P0 = ((yc + 0) * (SEGMENTS + 1) + (xc + 0))*3
          P1 = ((yc + 0) * (SEGMENTS + 1) + (xc + 1))*3
          P2 = ((yc + 1) * (SEGMENTS + 1) + (xc + 1))*3
          P3 = ((yc + 1) * (SEGMENTS + 1) + (xc + 0))*3
          A=neV3(P(p0),P(p0+1),P(p0+2))
          B=neV3(P(p1),P(p1+1),P(p1+2))
          C=neV3(P(p2),P(p2+1),P(p2+2))
          D=neV3(P(p3),P(p3+1),P(p3+2))

          n=a.normalized() : glNormal3fv(n) : glVertex3fv(A)
          n=b.normalized() : glNormal3fv(n) : glVertex3fv(B)
          n=d.normalized() : glNormal3fv(n) : glVertex3fv(D)
          
          n=b.normalized() : glNormal3fv(n) : glVertex3fv(B)
          n=c.normalized() : glNormal3fv(n) : glVertex3fv(C)
          n=d.normalized() : glNormal3fv(n) : glVertex3fv(D)
        Next
      Next
    glEnd()
  glEndList()
  return list
end function                

'  ########################
' # create a 1x2 capsule #
'######################## 
sub renderer.CreateCapsule(byref top    as GLuint, _
                           byref body   as GLuint, _
                           byref bottom as GLuint)
  dim as single l,r
  dim as integer i,j,n,ret
  dim as single tmp,a,ca,sa
  dim as single nx ,ny ,nz ,start_nx ,start_ny
  dim as single nx2,ny2,nz2,start_nx2,start_ny2
 
  ' number of sides to the cylinder (divisible by 4):
  n = 40
  l = 0.5
  r = 0.5
  a  = (M_PI*2.0)/n
  sa = sin(a)
  ca = cos(a)

  ' draw capsule body
  nx=0:ny=1:nz=0 ' normal vector
  body=glGenLists(1)
  glNewList(body, GL_COMPILE)
  glBegin (GL_TRIANGLE_STRIP)
  for i=0 to n 
    glNormal3f(ny,0,nz) : glVertex3f(ny*r,-l,nz*r) 
    glNormal3f(ny,0,nz) : glVertex3f(ny*r, l,nz*r)
    ' rotate ny,nz
    tmp = ca*ny - sa*nz
    nz  = sa*ny + ca*nz
    ny  = tmp
  next
  glEnd()
  glEndList() 
 
  top=glGenLists(1)
  glNewList (top, GL_COMPILE)
  ' draw capsule top cap
  start_nx = 0
  start_ny = 1
  for j=0 to (n/4)-1
    ' get start_n2 = rotated start_n
    start_nx2 =  ca*start_nx + sa*start_ny
    start_ny2 = -sa*start_nx + ca*start_ny
    ' get n=start_n and n2=start_n2
    nx = start_nx : ny  = start_ny : nz = 0
    nx2= start_nx2: ny2 = start_ny2: nz2= 0
    glBegin (GL_TRIANGLE_STRIP)
    for i= 0 to n
      glNormal3f(ny ,nx ,nz ) : glVertex3f(ny *r,nx *r-0.25,nz *r)
      glNormal3f(ny2,nx2,nz2) : glVertex3f(ny2*r,nx2*r-0.25,nz2*r)
      ' rotate n,n2
      tmp = ca*ny - sa*nz
      nz  = sa*ny + ca*nz
      ny  = tmp
      tmp = ca*ny2 - sa*nz2
      nz2 = sa*ny2 + ca*nz2
      ny2 = tmp
    next
    glEnd()
    start_nx = start_nx2
    start_ny = start_ny2
  next   
  glEndList() 

  bottom=glGenLists(1)
  glNewList (bottom, GL_COMPILE)
  ' draw capsule bottom cap
  start_nx = 0
  start_ny = 1
  for j=0 to (n/4)-1
    ' get start_n2 = rotated start_n
    start_nx2 = ca*start_nx - sa*start_ny
    start_ny2 = sa*start_nx + ca*start_ny
    ' get n=start_n and n2=start_n2
    nx = start_nx: ny = start_ny: nz = 0
    nx2 = start_nx2: ny2 = start_ny2: nz2 = 0
    glBegin (GL_TRIANGLE_STRIP)
    for i=0 to n
      glNormal3f (ny2,nx2,nz2) : glVertex3f (ny2*r,nx2*r+0.25,nz2*r)
      glNormal3f (ny ,nx ,nz ) : glVertex3f (ny *r,nx *r+0.25,nz *r)
      
      ' rotate n,n2
      tmp = ca*ny - sa*nz
      nz  = sa*ny + ca*nz
      ny  = tmp
      tmp = ca*ny2 - sa*nz2
      nz2 = sa*ny2 + ca*nz2
      ny2 = tmp
    next
    glEnd()
    start_nx = start_nx2
    start_ny = start_ny2
  next
  glEndList() 
end sub

sub Renderer.Render(byval g as neGeometry ptr)
  dim as single GeoMat(15)=any
'  dim as neV3 v  =any
  dim as single fDiameter=any,fWidth=any,fHeight=any,fDepth
  if (g=0) then return
  glPushMatrix()
    GeometryGetOpenGLMatrix(g,@GeoMat(0))
    glMultMatrixf(@GeoMat(0))
    if GeometryGetBoxSize(g,fWidth,fHeight,fDepth) then
      glPushMatrix() ' draw a box
        glScalef(fWidth, fHeight, fDepth)
        glCallList(BoxList)
      glPopMatrix()
    elseif GeometryGetSphereDiameter(g,fDiameter)  then  
      glPushMatrix() ' draw a sphere
        glScalef(fDiameter,fDiameter,fDiameter)
        glCallList(SphereList)
      glPopMatrix()
    elseif GeometryGetCapsule(g,fDiameter,fHeight) then  
      dim as single fWidth=fDiameter*0.25+fHeight/2
      glPushMatrix() ' draw a capsule
        glTranslatef(0,fWidth,0)
        glScalef(fDiameter, fDiameter, fDiameter)
        glCallList(CapsuleTop)
      glPopMatrix()
      glPushMatrix()
        glScalef(fDiameter, fHeight, fDiameter)
        glCallList(CapsuleBody) 
      glPopMatrix()
      glPushMatrix()
        glTranslatef(0,-fWidth,0)
        glScalef(fDiameter, fDiameter, fDiameter)
        glCallList(CapsuleBottom)
      glPopMatrix()
    end if
  glPopMatrix()  
end sub

sub Renderer.Render(byval s as neSensor ptr)
  dim as neV3 p=any,d=any
  if s=0 then return
  ' draw a line
  p = SensorGetLinePositionV3(s)
  d = SensorGetLineVectorV3(s)
  glColor3f(.8,.8,.2)
  glDisable(GL_LIGHTING)
  glLineWidth(3)
  glBegin(GL_LINES)
    glVertex3fv(p)
    p.x+=d.x : p.y+=d.y : p.z+=d.z
    glVertex3fv(p)
  glEnd()
  glLineWidth(1)
  glEnable(GL_LIGHTING)
end sub

' ToDo: 
sub Renderer.Render(byval j as neJoint ptr)
  if j=0 then return
  dim as neV3 posA,posB 
  var rba = JointGetRigidBodyA(j)
  var rbb = JointGetRigidBodyB(j)
  var abb = JointGetAnimatedBodyB(j)
  if rba<>0 andalso rbb<>0 then
    posA = RigidBodyGetPositionV3(rba)
    posB = RigidBodyGetPositionV3(rba)
  elseif rba<>0 andalso abb<>0 then
    posA = RigidBodyGetPositionV3(rba)
    posB = AnimatedBodyGetPositionV3(abb)
  elseif rba<>0 then
    posA = RigidBodyGetPositionV3(rba)
    var t=JointGetFrameB(j)
    posB=t.pos
  else
    return
  end if  
  glColor3f(.8,.2,.8)
  glDisable(GL_LIGHTING)
  glBegin(GL_LINES)
    glVertex3fv(posA)
    glVertex3fv(posB)
  glEnd()
  glEnable(GL_LIGHTING)
    
  var t = JointGetType(j)
  select case as const t
  case NE_JOINT_BALLSOCKET
  case NE_JOINT_HINGE
  case NE_JOINT_SLIDE
  end select

end sub

sub Renderer.Render(byval ab as neAnimatedBody ptr, byval r as f32,byval g as f32,byval b as f32, byval drawSensors as boolean)
  dim as single BodyMat(15)=any
  if ab=0 then return
  glPushMatrix()
    AnimatedBodyGetOpenGLMatrix(ab,@BodyMat(0))
    glMultMatrixF(@BodyMat(0))
    glColor3f(r,g,b)
    AnimatedBodyBeginIterateGeometry(ab)
    var ge = AnimatedBodyGetNextGeometry(ab)
    while ge
      Render(ge) : ge = AnimatedBodyGetNextGeometry(ab)
    wend
    if drawSensors then
      AnimatedBodyBeginIterateSensor(ab)
      var se = AnimatedBodyGetNextSensor(ab)
      while se
        Render(se) : se = AnimatedBodyGetNextSensor(ab)
      wend  
    end if  
  glPopMatrix()
end sub

sub Renderer.Render(byval ab as neAnimatedBody ptr, byval drawSensors as boolean)
  if ab=0 then return
  if AnimatedBodyGetActive(ab) then
    Render(ab,.2,.2,.8,drawSensors)
  else   
    Render(ab,.1,.1,.4,drawSensors)
  end if  
end sub


sub Renderer.Render(byval rb as neRigidBody ptr, byval r as f32,byval g as f32,byval b as f32, byval drawSensors as boolean)
  dim as single BodyMat(15)=any
  if rb=0 then return
  glPushMatrix()
    RigidBodyGetOpenGLMatrix(rb,@BodyMat(0))
    glMultMatrixF(@BodyMat(0))
    glColor3f(r,g,b)
    RigidBodyBeginIterateGeometry(rb)
    var geo = RigidBodyGetNextGeometry(rb)
    while geo
      Render(geo) : geo = RigidBodyGetNextGeometry(rb)
    wend
    if drawSensors then
      RigidBodyBeginIterateSensor(rb)
      var sen = RigidBodyGetNextSensor(rb)
      while sen
        Render(sen) : sen = RigidBodyGetNextSensor(rb)
      wend  
    end if  
  glPopMatrix()
end sub  

sub Renderer.Render(byval rb as neRigidBody ptr, byval drawSensors as boolean)
  if rb=0 then return
  if RigidBodyGetActive(rb) then
    if RigidBodyIsIdle(rb) then
      Render(rb, .8,.2,.2,drawSensors)
    else
      Render(rb, .2,.8,.2,drawSensors)
    end if  
  else
    Render(rb, .4,.1,.1,drawSensors)
  end if  
end sub
               
sub Renderer.Render(byref t as neTriangleMesh)
  if t.List=0 then
    if t.vertexCount  <3 or t.vertices =0 or _
       t.triangleCount<1 or t.triangles=0 then return
    t.List=glGenLists(1)
    glNewList(t.list, GL_COMPILE)
    
    glBegin (GL_TRIANGLES)
    for i as integer = 0 to t.triangleCount-1
      dim as neV3 a = t.vertices[t.triangles[i].indices(0)]
      dim as neV3 b = t.vertices[t.triangles[i].indices(1)]
      dim as neV3 c = t.vertices[t.triangles[i].indices(2)]
      dim as neV3 n = ((b-a)\(c-a)).Normalized() 
      glColor3f(.5, .25, .125)
      glNormal3fv(n)
      glVertex3fv(a)
      glVertex3fv(b)
      glVertex3fv(c)
#if 0 ' no back faces
      n*=-1:
      glNormal3fv(n)
      glVertex3fv(c)
      glVertex3fv(b)
      glVertex3fv(a)
#endif
    next  
    glEnd()
    glEndList()
  end if
  if t.List then glCallList(t.List)
end sub  

#endif ' __redering_bi__
