#include once "rendering.bi"

' test of:
'  terrain.AddVertex
'  terrain.AddTriangle
'  SimulatorSetTerrainMesh



const TSIZE=64
const TMASK=TSIZE-1
dim as neV3 points(TMASK,TMASK)

dim as RENDERER gfx 

' a grid of points
for z as integer = 0 to TMASK
  for x as integer = 0 to TMASK
    points(x,z).x=-TSIZE/2+x
    points(x,z).y=(rnd-rnd)*20
    points(x,z).z=-TSIZE/2+z
  next
next

' smooth the random y coords
for i as integer = 1 to 20
  for z as integer = 0 to TMASK
    dim as integer zp = (z+1) mod TMASK
    for x as integer = 0 to TMASK
      dim as integer xp = (x+1) mod TMASK
      points(x,z).y=(points( x,z ).y _
                    +points(xp,z ).y _
                    +points( x,zp).y _
                    +points(xp,zp).y)*0.25
    next
  next
next

for z as integer = 0 to TMASK
  for x as integer = 0 to TMASK
    if x=0 or x=TMASK or z=0 or z=TMASK then  points(x,z).y=10
  next
next



windowtitle "build terrain ..."
' build tarrain from smoothed grid
dim as neTriangleMesh terrain
for z as integer = 0 to TMASK-1
  for x as integer = 0 to TMASK-1
    var a = terrain.AddVertex(points(x,z))
    var b = terrain.AddVertex(points(x,z+1))
    var c = terrain.AddVertex(points(x+1,z+1))
    var d = terrain.AddVertex(points(x+1,z))
    var t1 = terrain.AddTriangle(a,b,c)
    var t2 = terrain.AddTriangle(c,d,a)
  next
next

windowtitle "create simulator ..."
const NBODIES=200
dim as neSimulatorSizeInfo SizeInfo
with SizeInfo
  .rigidBodiesCount     = NBODIES
  .animatedBodiesCount  = 1
  .geometriesCount      = .rigidBodiesCount + .animatedBodiesCount
  .overlappedPairsCount = .geometriesCount*4
end with
var sim = CreateSimulator(SizeInfo)
SimulatorSetTerrainMesh(sim,@terrain)

windowtitle "create bodies ..."
dim as neRigidBody ptr bodies(NBODIES-1)
for i as integer = 0 to NBODIES-1
  if i and 1 then
    if rnd<0.5 then
      bodies(i) = CreateRigidBodySphere(sim,.5+rnd,100)
    else
      bodies(i) = CreateRigidBodyCapsule(sim,.5+rnd,.5+rnd,100)
    end if  
    RigidBodySetAngularDamping(bodies(i),0.1)
  else
    if rnd<0.5 then
      bodies(i) = CreateRigidBodyCube(sim,.5+rnd,100)
    else
      bodies(i) = CreateRigidBodyBox(sim,.5+rnd,.5+rnd,.5+rnd,100)
    end if  
  end if  
  RigidBodySetLinearDamping(bodies(i),0.01)
  RigidBodySetPositionRotation(bodies(i), _
                              (rnd-rnd)*TSIZE/2*0.9,30+i,(rnd-rnd)*TSIZE/2*0.9, _
                              rnd*360,rnd*360,rnd*360)
next

windowtitle "ok ..."

dim as integer frame
dim as f32 x,y,z,w
var tStart=timer
while inkey()=""
  SimulatorAdvance(sim)
  if frame mod 2=0 then
    var tNow = (timer()-tStart)*0.25
    x=cos(tNow)*TSIZE/2
    z=sin(tNow)*TSIZE/2
    y=TSIZE/2+(x+z)*0.5
    glLoadIdentity() : LookAt(x,y,z, 0,0,0, 0,1,0)
    gfx.cls
    gfx.Render(terrain)
    for i as integer = 0 to NBODIES-1
      gfx.Render(bodies(i), .8,.8,.8)
    next  
    gfx.SwapBuffers()
  else
    sleep(10) ' don't use all CPU cycles
  end if
  frame+=1
wend

DestroySimulator(sim)

