#ifndef __tokamak_c_bi__
#define __tokamak_c_bi__

#include once "crt/limits.bi"
#include once "crt/math.bi"

#ifndef NULL
#define NULL cptr(any ptr,0)
#endif

' FreeBASIC: 
' optional move *.bi to the FreeBASIC/inc folder

' Windows optional:
' move lib/win32/tokamak-c-32.dll.a to FreeBASIC/lib/win32 folder
' move lib/win64/tokamak-c-64.dll.a to FreeBASIC/lib/win64 folder

' Linux: 
' move tokamak-c-32.so or tokamak-c-64.so to /usr/lib

#ifndef __FB_64BIT__
 #inclib "tokamak-c-32"  
#else
 #inclib "tokamak-c-64"  
#endif

type u32 as ulong
type s32 as long
type f32 as single

#define NE_PI M_PI
#define NE_ZERO (1.0e-6f)
#define neAbs(x) iif(x<0,-x,x)
#define neIsConsiderZero(f) iif(neAbs(f) < NE_ZERO, true, false)


type neV3
  declare constructor(byval x_ as f32=0.0f,byval y_ as f32=0.0f,byval z_ as f32=0.0f)
  declare operator cast as f32 ptr
  declare operator +=(byref r as const neV3)
  declare operator -=(byref r as const neV3)
  declare operator *=(byval r as f32)
  declare function Length as f32
  declare sub      Normalize
  declare function Normalized as neV3
  as f32 x=any,y=any,z=any,pad=any
end type
declare operator -(byref l as const neV3) as neV3
declare operator +(byref l as const neV3, byref r as const neV3) as neV3
declare operator -(byref l as const neV3, byref r as const neV3) as neV3
declare operator *(byref l as const neV3, byval r as f32) as neV3
declare operator *(byref l as const neV3, byref r as const neV3) as f32
declare operator \(byref l as const neV3, byref r as const neV3) as neV3

constructor neV3(byval x_ as f32,byval y_ as f32,byval z_ as f32)
  x=x_ : y=y_ : z=z_
end constructor

operator neV3.cast as f32 ptr
  operator = @this.x
end operator

operator neV3.+=(byref r as const neV3)
  x+=r.x : y+=r.y : z+=r.z 
end operator
operator neV3.-=(byref r as const neV3)
  x-=r.x : y-=r.y : z-=r.z 
end operator
operator neV3.*=(byval r as f32)
  x*=r : y*=r : z*=r 
end operator

operator -(byref l as const neV3) as neV3
  operator = type<neV3>(-l.x, -l.y, -l.z)
end operator
operator +(byref l as const neV3, byref r as const neV3) as neV3
  operator = type<neV3>(l.x+r.x, l.y+r.y, l.z+r.z)
end operator
operator -(byref l as const neV3, byref r as const neV3) as neV3
  operator = type<neV3>(l.x-r.x, l.y-r.y, l.z-r.z)
end operator
operator *(byref l as const neV3, byval r as f32) as neV3
  operator = type<neV3>(l.x*r, l.y*r, l.z*r)
end operator
operator *(byref l as const neV3, byref r as const neV3) as f32
  operator = l.x*r.x + l.y*r.y + l.z*r.z
end operator
operator \(byref l as const neV3, byref r as const neV3) as neV3
  operator = type<neV3>(l.y*r.z - l.z*r.y, l.z*r.x - l.x*r.z, l.x*r.y - l.y*r.x)
end operator

function neV3.Length as f32
  var l = x*x + y*y + z*z
  if neIsConsiderZero(l) then return 0.0f
  return sqrtf(l)
end function
sub neV3.Normalize
  var l = x*x + y*y + z*z
  if neIsConsiderZero(l) then return
  l=1.0f/sqrtf(l) : x*=l : y*=l : z*=l
end sub
function neV3.Normalized as neV3
  var l = x*x + y*y + z*z
  if neIsConsiderZero(l) then return type<neV3>(0.0f,0.0f,0.0f)
  if l then l=1.0f/sqrtf(l)
  return type<neV3>(x*l, y*l, z*l)
end function


type neQ
  as f32 x,y,z,w
end type

type neM3
  as neV3 M(2)
end type

type neT3
  as neM3 rot
  as neV3 pos
end type

const as s32 DEFAULT_RIGIDBODIES_COUNT          = 50   ' (50)
const as s32 DEFAULT_ANIMATEDBODIES_COUNT       = 50   ' (50)
const as s32 DEFAULT_RIGIDPARTICLES_COUNT       = 50   ' (50)
const as s32 DEFAULT_GEOMETRIES_COUNT           = 100  ' (RIGIDBODIES_COUNT + ANIMATEDBODIES_COUNT)
const as s32 DEFAULT_OVERLAPPED_PAIRS_COUNT     = 1225 ' (GEOMETRIES_COUNT*(GEOMETRIES_COUNT-1))\2
const as s32 DEFAULT_CONTROLLERS_COUNT          = 50   ' (50)
const as s32 DEFAULT_SENSORS_COUNT              = 100  ' (100)
const as s32 DEFAULT_CONSTRAINTS_COUNT          = 100  ' (100)
const as s32 DEFAULT_CONTRAINT_SETS_COUNT       = 100  ' (100)
const as s32 DEFAULT_CONTRAINT_BUFFER_SIZE      = 2000 ' (2000)
const as s32 DEFAULT_TERRAIN_NODES_START_COUNT  = 200  ' (200)
const as s32 DEFAULT_TERRAIN_NODES_GROWBY_COUNT = -1

type neSimulatorSizeInfo
  ' a neSimulatorSizeInfo are used to init the neSimulator
  declare constructor
  ' Number of rigid bodies in the simulation
  as s32 rigidBodiesCount
  ' Number of animated bodies in the simulation
  as s32 animatedBodiesCount
  ' Number of rigid particles in the simulation
  as s32 rigidParticleCount
  ' Number of controller instances in the simulation
  as s32 controllersCount
  ' Number of possible overlapping pairs.
  ' This has the maximum value of (n x (n - 1)) / 2, where n = rigidBodyCount + animatedBodyCount.
  ' But in practice it rarely reach that high. You can try to specify a smaller number to save memory.
  as s32 overlappedPairsCount
  ' Number of collision geometries in the simulator
  as s32 geometriesCount
  ' Number of joints in the simulation
  as s32 constraintsCount
  ' Number of joint Sets in the simulation
  as s32 constraintSetsCount
  ' Size of the buffer use to solve joints
  as s32 constraintBufferSize
  ' Number of sensors in the simulation
  as s32 sensorsCount
  ' Number of nodes use to store terrain triangles
  as s32 terrainNodesStartCount
  ' Grow by this size if run out of nodes
  ' -1 means: signify double the number of terrainNode, whenever the it reach full capacity.
  as s32 terrainNodesGrowByCount
end type

constructor neSimulatorSizeInfo
  ' The constructor function initialises the members to the default values.
  rigidBodiesCount        = DEFAULT_RIGIDBODIES_COUNT
  animatedBodiesCount     = DEFAULT_ANIMATEDBODIES_COUNT
  rigidParticleCount      = DEFAULT_RIGIDPARTICLES_COUNT
  controllersCount        = DEFAULT_CONTROLLERS_COUNT
  overlappedPairsCount    = DEFAULT_OVERLAPPED_PAIRS_COUNT
  geometriesCount         = DEFAULT_GEOMETRIES_COUNT
  constraintsCount        = DEFAULT_CONSTRAINTS_COUNT
  constraintSetsCount     = DEFAULT_CONTRAINT_SETS_COUNT
  constraintBufferSize    = DEFAULT_CONTRAINT_BUFFER_SIZE
  sensorsCount            = DEFAULT_SENSORS_COUNT
  terrainNodesStartCount  = DEFAULT_TERRAIN_NODES_START_COUNT
  terrainNodesGrowByCount = DEFAULT_TERRAIN_NODES_GROWBY_COUNT
end constructor

type neSimulator
  ' a neSimulator solves forces on bodies and handle collision between collision shapes
  as integer dummy
end type

type neCollisionTable
  ' Using the collision table, the application can specify the response whenever two bodies collide in the simulation.
  ' Each body in the simulation has a collision ID, which defaults to 0.
  ' A collision table contains a neResponseBitFlag for each pair of collision IDs.
  ' A neResponseBitFlag equal to RESPONSE_IGNORE means the collision will be ignored; no collision impulse or collision callback will be performed.
  ' The application can retrieve the neCollisionTable by calling neSimulatorGetCollisionTable function.
  as integer dummy
end type
type neReponseBitFlag as u32
const as neReponseBitFlag RESPONSE_IGNORE           = 0
const as neReponseBitFlag RESPONSE_IMPULSE          = 1
const as neReponseBitFlag RESPONSE_CALLBACK         = 2
const as neReponseBitFlag RESPONSE_IMPULSE_CALLBACK = 3

type neGeometry
  ' a neGeometry descripts a collision shape and used by neAnimatedBody and neRigidBody
  ' (note: it exists one more collision shape a terrain)
  as integer dummy
end type
type GEOMETRY_TYPE as u32
const as GEOMETRY_TYPE GEOMETRY_BOX     = 3
const as GEOMETRY_TYPE GEOMETRY_SPHERE  = 4
const as GEOMETRY_TYPE GEOMETRY_CAPSULE = 5

type neAnimatedBody
  ' it's normaly a static massless object with a collision neGeometry like a building ...
  ' and will never moved by the simulator (after a collision with a neRigidBody for example)
  ' "Animated" in this scope are a little bit misspelled
  ' however it means it can be animated (moved and or rotated) only by user
  ' a neAnimatedBody are fixed in space during a simulation step and can't collide with an other neAnimatedBody
  ' (note: you can define a geomtry as breakable if a defined limit of forces act on 
  ' it will be split in parts and converted to neRigidBody)
  as integer dummy
end type
type neBreakFlag as u32
' The geometry does not break off upon collision
const as neBreakFlag NE_BREAK_DISABLE            = 0
' Normal breakage mode. Only this geometry will break when a suitably large collision has occurred.
const as neBreakFlag NE_BREAK_NORMAL             = 1
' When this geometry breaks off, all geometries within the current object, with break the flag not equal to NE_BREAK_DISABLE, will also break off.
const as neBreakFlag NE_BREAK_ALL                = 2
'  When this geometry breaks off, its neighbours within a certain radius will also break off. 
' This radius is specified by neGeometrySetBreakageNeighbourRadius.
const as neBreakFlag NE_BREAK_NEIGHBOUR          = 3
' the following are the same as the above, except that they specify the new object spawned from the geometry 
' will be a rigid particle, rather than a rigid body.
const as neBreakFlag NE_BREAK_NORMAL_PARTICLE    = 4
const as neBreakFlag NE_BREAK_ALL_PARTICLE       = 5
const as neBreakFlag NE_BREAK_NEIGHBOUR_PARTICLE = 6

type neRigidBody
  ' a neRgidBody has a mass and a collision neGeometry and will interact with all other rigid and animated bodies
  ' (one exption if a neRigidBody are created as a particle it will never collide with other particles)
  as integer dummy
end type

type neRigidBodyController
  ' a neRigidBodyController are used inside a optional user defined neRigidBodyCallback
  as integer dummy
end type

type neJoint
  ' a neJoint descipts the constrain of agility of an neRigidBody or the contrains between neRigidBodies
  ' (note the NE_JOINT_HINGE is the only joint that can be used as a motor axis)
  as integer dummy
end type

type neJointController
  ' a joint controller are used inside a optional user defined joint callback
  as integer dummy
end type

type neSensor
  ' neSensor is a line with position, length and a directition and can be added to neAnimated and neRigidBody.
  ' When the line representing the neSensor intersects with other bodies neGeometry shapes and the terrain (if any) 
  ' the intersection information can be retrieved and used by the application.
  as integer dummy 
end type

type neBodyType as u32
const as neBodyType NE_TERRAIN       = 0
const as neBodyType NE_RIGID_BODY    = 1
const as neBodyType NE_ANIMATED_BODY = 2

' This structure is passed to the application during neCollisionCallback.
' It contains various blocks of information about a collision which has occurred during the current simulation step.
type neCollisionInfo
  union
  ' bodyA and bodyB are pointers to the bodies which are colliding.
  ' Their type depends on the value of typeA and typeB.
  ' If the value of typeB is NE_TERRAIN, then bodyB is NULL.
  as neRigidBody    ptr RigidBodyA
  as neAnimatedBody ptr AnimatedBodyA 
  end union
  union
  as neRigidBody    ptr RigidBodyB
  as neAnimatedBody ptr AnimatedBodyB 
  end union
  as neBodyType typeA
  as neBodyType typeB
  ' These are pointers to neGeometry objects involved in the collision, one from each of the body.
  ' If typeB equal to NE_TERRAIN, then geometryB is NULL
  as neGeometry ptr geometryA
  as neGeometry ptr geometryB
  ' The material indices of the geometries which are colliding.
  as s32 materialIdA
  as s32 materialIdB
  ' The contact points in body coordinates and in world coordinates.
  as neV3 bodyContactPointA    ' contact point A in body space of A
  as neV3 bodyContactPointB    ' contact point B in body space of B
  as neV3 worldContactPointA   ' contact point A in world space
  as neV3 worldContactPointB   ' contact point B in world space
  ' The relativeVelocity of the contact points. 
  as neV3 relativeVelocity
  ' The direction of the collision normal.
  as neV3 collisionNormal
end type

type neTraingleflag as u32
const as neTraingleflag NE_TRI_TRIANGLE   = 0
const as neTraingleflag NE_TRI_HEIGHT_MAP = 1

type neTriangle
  ' a neTriangle is the shortes pice of a neTriangleMesh
  as s32 indices(2)
  as s32 materialID
  as neTraingleflag flag
  as any ptr userData
end type

type neTriangleMesh
  ' a neTriangleMesh is a collection of neTriangles and are used to describe a terrain
  ' the terrain are internaly stored in a quadtree the tree nodes are really fast 
  ' to detect collision with neRigidBodies and neSensors
  ' this makes it perfect to describe many static parts of your simualted world. 
  ' (ground, mountains, buildings, streets ...)
  declare function AddVertex(byval x as f32, _
                             byval y as f32, _
                             byval z as f32) as integer
  declare function AddVertex(byref v as const neV3) as integer
  declare function AddTriangle(byval a as integer, _
                               byval b as integer, _
                               byval c as integer) as integer
  as neV3 ptr       vertices
  as s32            vertexCount
  as neTriangle ptr triangles
  as s32            triangleCount
  as u32            List
end type
function neTriangleMesh.AddVertex(byval x as f32, _
                                  byval y as f32, _
                                  byval z as f32) as integer
  #if 1
  if vertexCount>0 andalso vertices<>0 then
    for i as integer=0 to vertexCount-1
      if vertices[i].x=x andalso _
         vertices[i].y=y andalso _
         vertices[i].z=z then return i
    next
  end if
  #endif
  var index = vertexCount : vertexCount+=1
  vertices=reallocate(vertices,vertexCount*sizeof(neV3))
  vertices[index].x=x
  vertices[index].y=y
  vertices[index].z=z
  return index
end function
function neTriangleMesh.AddVertex(byref v as const neV3) as integer
  return AddVertex(v.x,v.y,v.z) 
end function   
function neTriangleMesh.AddTriangle(byval a as integer, _
                                    byval b as integer, _
                                    byval c as integer) as integer
  #if 1
  if triangleCount>0 andalso triangles<>0 then
    for i as integer=0 to triangleCount-1
      if triangles[i].indices(0)=a andalso _
         triangles[i].indices(1)=b andalso _
         triangles[i].indices(2)=c then return i
    next
  end if
  #endif 
  var index = triangleCount : triangleCount+=1
  triangles=reallocate(triangles,triangleCount*sizeof(neTriangle))
  triangles[index].indices(0)=a
  triangles[index].indices(1)=b
  triangles[index].indices(2)=c
  triangles[index].materialID=0
  triangles[index].flag=NE_TRI_TRIANGLE
  triangles[index].userData=0  
  return index
end function  


type LOG_OUTPUT_LEVEL as u32
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_NONE = 0
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_ONE  = 1
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_FULL = 2

' ###################################
' # optinal: user defined callbacks #
' ###################################
type neCustomCDInfo
  ' the neCustomCDInfo struct must be filled inside of the optinal neCustomCDRB2RB and neCustomCDRB2AB callback's
  as neV3 collisionNormal
  as neV3 worldContactPointA
  as neV3 worldContactPointB
  as f32  penetrationDepth
  as s32  materialIdA
  as s32  materialIdB
end type

extern "c"

type neCustomCDRB2RBCallback        as function (byval bodyA  as neRigidBody ptr, _
                                                 byval bodyB  as neRigidBody ptr, _
                                                 byref cdInfo as neCustomCDInfo) as boolean

type neCustomCDRB2ABCallback        as function (byval bodyA  as neRigidBody ptr, _
                                                 byval bodyB  as neAnimatedBody ptr, _
                                                 byref cdInfo as neCustomCDInfo) as boolean

type neCollisionCallback            as sub      (byref collisionInfo as neCollisionInfo)

type neTerrainTriangleQueryCallback as sub      (byref minBound           as const neV3, _
                                                 byref maxBound           as const neV3, _
                                                 byval candidateTriangles as s32 ptr ptr, _
                                                 byval triangles          as neTriangle ptr ptr, _
                                                 byval vertices           as neV3 ptr ptr, _
                                                 byval candidateCount     as s32 ptr, _
                                                 byval triangleCount      as s32 ptr, _
                                                 byval body               as neRigidBody ptr)

type neBreakageCallback             as sub      (byval originalBody     as any ptr, _
                                                 byval originalbodyType as neBodyType, _
                                                 byval brokenGeometry   as neGeometry ptr, _
                                                 byval newBody          as neRigidBody)

type neRigidBodyControllerCallback  as sub      (byval controller as neRigidBodyController ptr, byval timeStep as f32)

type neJointControllerCallback      as sub      (byval controller as neJointController     ptr, byval timeStep as f32)

type neLogOutputCallback            as sub      (byval logString as zstring ptr)

' ###############
' # neSimulator #
' ###############
' Use CreateSimulator to create a new instance of the simulator. 
' sizeInfo specifies the amount of various objects to allocate within the simulator.
declare function CreateSimulator(byref sizeInfo    as const neSimulatorSizeInfo=type<neSimulatorSizeInfo>, _
                                 byref gravitation as const neV3=type<neV3>(0,-9.81,0)) as neSimulator ptr 
' Use DestroySimulator to delete an existing instance of the simulator object.
declare sub      DestroySimulator(byref sim as neSimulator ptr)


declare sub      SimulatorSetGravity(byval sim as neSimulator ptr, byval gx as f32, byval gy as f32, byval gz as f32)
declare sub      SimulatorGetGravity(byval sim as neSimulator ptr, byref gx as f32, byref gy as f32, byref gz as f32)

declare sub      SimulatorSetGravityV3(byval sim as neSimulator ptr, byref grav as const neV3)
declare function SimulatorGetGravityV3(byval sim as neSimulator ptr) as neV3


' Use these functions to create or free a rigid body (neRigidBody), rigid particle (neRigidBody), or animated body (neAnimatedBody).
' When a body is freed, all their associated joints, geometries, sensors and callbacks are freed automatically.
declare function SimulatorCreateRigidBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorCreateRigidParticle(byval sim as neSimulator ptr) as neRigidBody ptr
declare sub      SimulatorFreeRigidBody(byval sim as neSimulator ptr, byref body as neRigidBody ptr)

declare function SimulatorCreateAnimatedBody(byval sim as neSimulator ptr) as neAnimatedBody ptr
declare sub      SimulatorFreeAnimatedBody(byval sim as neSimulator ptr, byref body as neAnimatedBody ptr)

' Use this function to specify the terrain mesh to the simulator.
declare sub      SimulatorSetTerrainMesh(byval sim as neSimulator ptr, byval tris as neTriangleMesh ptr)
declare sub      SimulatorFreeTerrainMesh(byval sim as neSimulator ptr)

' Specify and retrieve the material properties for a corresponding material index.
' By default all materials have the following values: friction = 0.4 restitution = 0.5
declare function SimulatorSetMaterial(byval sim as neSimulator ptr, byval index as s32, byval friction as f32, byval restitution as f32) as boolean
declare function SimulatorGetMaterial(byval sim as neSimulator ptr, byval index as s32, byref friction as f32, byref restitution as f32) as boolean

' Advance the simulation by the amount sec, specified in milliseconds. nSteps specifies the number of sub-steps to perform.
declare sub      SimulatorAdvance(byval sim as neSimulator ptr, byval sec as f32 = 1/60)
declare sub      SimulatorAdvanceSteps(byval sim as neSimulator ptr, byval sec as f32 = 1/60, byval steps as s32)
declare sub      SimulatorAdvanceMinMax(byval sim as neSimulator ptr, byval sec as f32 = 1/60, byval minTimeStep as f32, byval maxTimeStep as f32)

' Create and free a joint object.
' The first form of CreateJoint takes one neRigidBody.
' This means the joint constraint of the rigid body to the world.
' The second and third form require a joint which constrains bodyA to another rigid body or to an animated body respectively.
' When a rigid body or animated body is freed, all their associated joints, geometries, sensors and callbacks are freed automatically.
declare function SimulatorCreateRigidBodyWorldJoint(byval sim as neSimulator ptr, byval body as neRigidBody ptr) as neJoint ptr
declare function SimulatorCreateRigidBodyRigidBodyJoint(byval sim as neSimulator ptr, byval bodyA as neRigidBody ptr, byval bodyB as neRigidBody ptr) as neJoint ptr
declare function SimulatorCreateRigidBodyAnimatedBodyJoint(byval sim as neSimulator ptr, byval bodyA as neRigidBody ptr, byval bodyB as neAnimatedBody ptr) as neJoint ptr
declare sub      SimulatorFreeJoint(byval sim as neSimulator ptr, byref joint as neJoint ptr)

' simulator misc.
declare sub      SimulatorSetLogOutputLevel(byval sim as neSimulator ptr, byval lvl as LOG_OUTPUT_LEVEL)
declare function SimulatorGetCurrentSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo
declare function SimulatorGetStartSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo
declare sub      SimulatorGetMemoryAllocated(byval sim as neSimulator ptr, byref memoryAllocated as s32)

declare function SimulatorGetIdleBodyCount(byval sim as neSimulator ptr) as s32


' #######################
' # optional callback's #
' #######################
' Specify and retrieve the collision callback function.
' The collision callback function returns information about a collision between two objects in the simulation.
' Each pair of objects will only receive a maximum of one collision callback for each timestep of the simulation.
declare sub      SimulatorSetCollisionCallback(byval sim as neSimulator ptr, byval cb as neCollisionCallback)
declare function SimulatorGetCollisionCallback(byval sim as neSimulator ptr) as neCollisionCallback

' Specify and retrieve the breakage callback function.
' The breakage callback function is called by the simulator whenever a breakage is about to occur.
' The callback function is called for each geometry object which has broken.
' When the geometry is broken, a new rigid body (or rigid particle, depending the breakage flag) is spawned.
' During the breakage callback function the application can retrieve the pointer to the newly spawned object.
declare sub      SimulatorSetBreakageCallback(byval sim as neSimulator ptr, byval cb as neBreakageCallback)
declare function SimulatorGetBreakageCallback(byval sim as neSimulator ptr) as neBreakageCallback

declare sub      SimulatorSetTerrainTriangleQueryCallback(byval sim as neSimulator ptr, byval cb as neTerrainTriangleQueryCallback)
declare function SimulatorGetTerrainTriangleQueryCallback(byval sim as neSimulator ptr) as neTerrainTriangleQueryCallback

declare sub      SimulatorSetCustomCDRB2RBCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2RBCallback)
declare function SimulatorGetCustomCDRB2RBCallback(byval sim as neSimulator ptr) as neCustomCDRB2RBCallback

declare sub      SimulatorSetCustomCDRB2ABCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2ABCallback)
declare function SimulatorGetCustomCDRB2ABCallback(byval sim as neSimulator ptr) as neCustomCDRB2ABCallback

declare sub      SimulatorSetLogOutputCallback(byval sim as neSimulator ptr, byval cb as neLogOutputCallback)
declare function SimulatorGetLogOutputCallback(byval sim as neSimulator ptr) as neLogOutputCallback


' ####################
' # neCollisionTable #
' ####################
' Retrieve the pointer to the collision table.
' A collision table specifies the collision response between different collision groups inside the simulation.
declare function SimulatorGetCollisionTable(byval sim as neSimulator ptr) as neCollisionTable ptr
declare sub      CollisionTableSet(byval tbl as neCollisionTable ptr, byval collisionID1 as s32, byval collisionID2 as s32, byval response as neReponseBitFlag = RESPONSE_IMPULSE)
declare function CollisionTableGet(byval tbl as neCollisionTable ptr, byval collisionID1 as s32, byval collisionID2 as s32) as neReponseBitFlag
declare function CollisionTableGetMaxCollisionID(byval tbl as neCollisionTable ptr) as s32

' ##############
' # neGeometry #
' ##############
' Specify and retrieve the user data associate with the neGeometry object.
declare sub      GeometrySetUserData(byval geo as neGeometry ptr, byval userData as any ptr)
declare function GeometryGetUserData(byval geo as neGeometry ptr) as any ptr

declare function GeometryGetType(byval geo as neGeometry ptr) as GEOMETRY_TYPE

declare sub      GeometrySetCubeSize(byval geo as neGeometry ptr, byval fSize as f32)

declare sub      GeometrySetBoxSize(byval geo as neGeometry ptr, byval fWidth as f32, byval fHeight as f32, byval fDepth as f32)
declare function GeometryGetBoxSize(byval geo as neGeometry ptr, byref fWidth as f32, byref fHeight as f32, byref fDepth as f32) as boolean

declare sub      GeometrySetBoxSizeV3(byval geo as neGeometry ptr, byref vSize as const neV3)
declare function GeometryGetBoxSizeV3(byval geo as neGeometry ptr, byref vSize as neV3) as boolean

declare sub      GeometrySetSphereDiameter(byval geo as neGeometry ptr, byval fDiameter as f32)
declare function GeometryGetSphereDiameter(byval geo as neGeometry ptr, byref fDiameter as f32) as boolean

declare sub      GeometrySetCapsule(byval geo as neGeometry ptr, byval fDiameter as f32, byval fHeight as f32)
declare function GeometryGetCapsule(byval geo as neGeometry ptr, byref fDiameter as f32, byref fHeight as f32) as boolean


' Set and retrieve the transform of the geometry.
' The transform defines the geometry's position and orientation relative to its host body object.
' The transform is defined in the host body's space.
declare sub      GeometrySetTransform(byval geo as neGeometry ptr, byref t as neT3)
declare function GeometryGetTransform(byval geo as neGeometry ptr) as neT3

' Set and retrieve position and roation of the geometry. 
declare sub      GeometrySetPosition(byval geo as neGeometry ptr, byval x as f32, byval y as f32, byval z as f32) 
declare sub      GeometryGetPosition(byval geo as neGeometry ptr, byref x as f32, byref y as f32, byref z as f32) 

declare sub      GeometrySetPositionRotation(byval geo as neGeometry ptr, _
                                             byval px as f32, byval py as f32, byval pz as f32, _
                                             byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      GeometryGetPositionRotation(byval geo as neGeometry ptr, _
                                             byref px as f32, byref py as f32, byref pz as f32, _
                                             byref rx as f32, byref ry as f32, byref rz as f32)
declare sub      GeometrySetPositionRotationV3(byval geo as neGeometry ptr, _
                                               byref position as const neV3, _
                                               byref rotation as const neV3)
declare sub      GeometryGetPositionRotationV3(byval geo as neGeometry ptr, _
                                               byref position as neV3, _
                                               byref rotation as neV3)

' Set and retrieve the material index for the geometry.
' The material index refers to a material defined in the neSimulator using the function neSimulatorSetMaterial.
' During a collision, the materials of the two colliding geometries are used to calculate the correct collision response.
declare sub      GeometrySetMaterialIndex(byval geo as neGeometry ptr, byval index as s32)
declare function GeometryGetMaterialIndex(byval geo as neGeometry ptr) as s32

' Specify and retrieve the breaking behaviour of the geometry.
declare sub      GeometrySetBreakageFlag(byval geo as neGeometry ptr, byval flag as neBreakFlag)
declare function GeometryGetBreakageFlag(byval geo as neGeometry ptr) as neBreakFlag

' Specify and retrieve the mass and inertia mass tensor of the object spawned from the breakage.
declare sub      GeometrySetBreakageMass(byval geo as neGeometry ptr, byval mass as f32)
declare function GeometryGetBreakageMass(byval geo as neGeometry ptr) as f32
declare sub      GeometrySetBreakageInertiaTensor(byval geo as neGeometry ptr, byref tensor as const neV3)
declare function GeometryGetBreakageInertiaTensor(byval geo as neGeometry ptr) as neV3

' Specify and retrieve the magnitude of the collision impulse required to break this geometry.
declare sub      GeometrySetBreakageMagnitude(byval geo as neGeometry ptr, byval mag as f32)
declare function GeometryGetBreakageMagnitude(byval geo as neGeometry ptr) as f32

' Specify and retrieve the amount of absorption the breaking of this geometry has.
declare sub      GeometrySetBreakageAbsorption(byval geo as neGeometry ptr, byval absorption as f32)
declare function GeometryGetBreakageAbsorption(byval geo as neGeometry ptr) as f32

' Specify and retrieve the radius within which other geometries within the same body will also break when a breakage occurs.
declare sub      GeometrySetBreakageNeighbourRadius(byval geo as neGeometry ptr, byval radius as f32)
declare function GeometryGetBreakageNeighbourRadius(byval geo as neGeometry ptr) as f32

declare sub      GeometrySetBreakagePlane(byval geo as neGeometry ptr, byref planeNormal as const neV3)
declare function GeometryGetBreakagePlane(byval geo as neGeometry ptr) as neV3

declare sub      GeometryGetOpenGLMatrix (byval geo as neGeometry ptr, byval m4x4 as f32 ptr)
declare sub      GeometryGetOpenGLMatrixInverted(byval geo as neGeometry ptr, byval m4x4 as f32 ptr)
declare function GeometryRotateVector    (byval geo as neGeometry ptr, byref v as const neV3) as neV3
declare function GeometryTransformVector (byval geo as neGeometry ptr, byref v as const neV3) as neV3
declare sub      GeometryRotateVectors   (byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)
declare sub      GeometryTransformVectors(byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)

' ##################
' # neAnimatedBody #
' ##################
' Specify and retrieve the user data associate with the neAnimatedBody object.
declare sub      AnimatedBodySetUserData(byval ab as neAnimatedBody ptr, byval userData as any ptr)
declare function AnimatedBodyGetUserData(byval ab as neAnimatedBody ptr) as any ptr

' Set and retrieve spatial states for the animated body with these functions.
declare sub      AnimatedBodySetPosition  (byval ab as neAnimatedBody ptr, byval x as f32, byval y as f32, byval z as f32)
declare sub      AnimatedBodyGetPosition  (byval ab as neAnimatedBody ptr, byref x as f32, byref y as f32, byref z as f32)
declare sub      AnimatedBodySetPositionV3(byval ab as neAnimatedBody ptr, byref position as const neV3)
declare function AnimatedBodyGetPositionV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve position and roation of the animated body. (rotation in rad)
declare sub      AnimatedBodySetRotation  (byval ab as neAnimatedBody ptr, byval p as f32, byval y as f32, byval r as f32)
declare sub      AnimatedBodyGetRotation  (byval ab as neAnimatedBody ptr, byref p as f32, byref y as f32, byref r as f32)
declare sub      AnimatedBodySetRotationV3(byval ab as neAnimatedBody ptr, byref pyr as neV3)
declare function AnimatedBodyGetRotationV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve roation of the animated body b<uaternion and matrix.
declare sub      AnimatedBodySetRotationM3(byval ab as neAnimatedBody ptr, byref m as const neM3)
declare function AnimatedBodyGetRotationM3(byval ab as neAnimatedBody ptr) as neM3
declare sub      AnimatedBodySetRotationQ (byval ab as neAnimatedBody ptr, byref q as const neQ)
declare function AnimatedBodyGetRotationQ (byval ab as neAnimatedBody ptr) as neQ

' Set and retrieve position and roation of the animated body. (rotation in degree)
declare sub      AnimatedBodySetPositionRotation  (byval ab as neAnimatedBody ptr, _
                                                   byval px as f32, byval py as f32, byval pz as f32, _
                                                   byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      AnimatedBodyGetPositionRotation  (byval ab as neAnimatedBody ptr, _
                                                   byref px as f32, byref py as f32, byref pz as f32, _
                                                   byref rx as f32, byref ry as f32, byref rz as f32)
declare sub      AnimatedBodySetPositionRotationV3(byval ab as neAnimatedBody ptr, _
                                                   byref position as const neV3, _
                                                   byref rotation as const neV3)
declare sub      AnimatedBodyGetPositionRotationV3(byval ab as neAnimatedBody ptr, _
                                                   byref position as neV3, _
                                                   byref rotation as neV3)

' Retrieve the transform of the geometry.
declare sub      AnimatedBodySetTransform(byval ab as neAnimatedBody ptr, byref t as const neT3)
declare function AnimatedBodyGetTransform(byval ab as neAnimatedBody ptr) as neT3

' Set and retrieve the collision ID of the animated body. 
' The collision ID, as32 with the neCollisionTable, determines how the animated body 
' will respond when a collision has occured with other objects in the simulation.
declare sub      AnimatedBodySetCollisionID(byval ab as neAnimatedBody ptr, byval cid as s32)
declare function AnimatedBodyGetCollisionID(byval ab as neAnimatedBody ptr) as s32

' Use these functions to create and retrieve neGeometry objects for the animated body.
' The collection neGeometry, bes32ing to an animated body, defines the body's collision boundary.
' It also defines the animated body's breaking behaviour. 
' An animated body can also exist without any neGeomtry assigned to it; it just wont collide with anything in the simulation.
' All neGeometry objects within an animated body will be freed automatcally when the animated body is freed. See also neGeometry.
declare function AnimatedBodyAddGeometry     (byval ab as neAnimatedBody ptr) as neGeometry ptr
declare function AnimatedBodyRemoveGeometry  (byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as boolean
declare function AnimatedBodyGetGeometryCount(byval ab as neAnimatedBody ptr) as s32
declare sub      AnimatedBodyBeginIterateGeometry(byval ab as neAnimatedBody ptr)
declare function AnimatedBodyGetNextGeometry(byval ab as neAnimatedBody ptr) as neGeometry ptr

' Update the internal bounding information of the animated body. 
' Whenever the user changes the geometry and/or sensor of the rigid body 
' by adding, removing, or modifying the geometry and/or sensor, 
' this function needs to be called to update the internal bounding information.
declare sub      AnimatedBodyUpdateBoundingInfo(byval ab as neAnimatedBody ptr)

declare function AnimatedBodyBreakGeometry(byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as neRigidBody  ptr

' Define whether or not an animated  body will collide with another body connected to it via joints (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable. 
declare sub      AnimatedBodySetCollideConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideConnected(byval ab as neAnimatedBody ptr) as boolean

declare sub      AnimatedBodySetCollideDirectlyConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideDirectlyConnected(byval ab as neAnimatedBody ptr) as boolean

declare sub      AnimatedBodySetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr, byval yes as boolean, byval obb as const neT3 ptr, byval boundingRadius as f32)
declare function AnimatedBodyGetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr) as boolean

' Use these functions to create and retrieve neSensor objects for the animated body. See neSensor.
declare function AnimatedBodyAddSensor(byval ab as neAnimatedBody ptr) as neSensor ptr
declare function AnimatedBodyRemoveSensor(byval ab as neAnimatedBody ptr, byval sen as neSensor ptr) as boolean
declare sub      AnimatedBodyBeginIterateSensor(byval ab as neAnimatedBody ptr)
declare function AnimatedBodyGetNextSensor(byval ab as neAnimatedBody ptr) as neSensor ptr

' Activate or deactivate the animated body. 
' When an animated body is deactivated, the simulator will not perform a collision test on the animated body.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the animated body being activated is.
' This can help speed up the introduction of the new activated animated body into the simulation.
declare sub      AnimatedBodySetActive(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetActive(byval ab as neAnimatedBody ptr) as boolean
declare sub      AnimatedBodySetActiveWithRigidBodyHint   (byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      AnimatedBodySetActiveWithAnimatedBodyHint(byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

declare sub      AnimatedBodyGetOpenGLMatrix(byval ab as neAnimatedBody ptr, byval m4x4 as f32 ptr)
declare sub      AnimatedBodyGetOpenGLMatrixInverted(byval ab as neAnimatedBody ptr, byval m4x4 as f32 ptr)
declare function AnimatedBodyRotateVector   (byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare function AnimatedBodyTransformVector(byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare sub      AnimatedBodyRotateVectors   (byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)
declare sub      AnimatedBodyTransformVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)

' ###############
' # neRigidBody #
' ###############
' Specify and retrieve the user data associate with the neRigidBody object.
declare sub      RigidBodySetUserData(byval rb as neRigidBody ptr, byval userData as any ptr)
declare function RigidBodyGetUserData(byval rb as neRigidBody ptr) as any ptr 

' Sets and retrieves the mass of the rigid body.
' This property governs how fast a rigid body will accelerate under a given force, 
' as well as how big an impulse is needed to change the body's velocity.
declare sub      RigidBodySetMass(byval rb as neRigidBody ptr, byval mass as f32)
declare function RigidBodyGetMass(byval rb as neRigidBody ptr) as f32

' Set the body's mass inertia tensor. 
' This property is analagous to mass, but effects the rotational behaviour. 
' Use utility functions such as neBoxInertiaTensor to help you set this value.
declare sub      RigidBodySetInertiaTensor  (byval rb as neRigidBody ptr, byref v as const neV3)
declare sub      RigidBodySetInertiaTensorM3(byval rb as neRigidBody ptr, byref m as const neM3)
declare sub      RigidBodyUpdateInertiaTensor(byval rb as neRigidBody ptr)

' Set and retrieve the collision ID of the rigid body.
' The collision ID, and with the neCollisionTable, determines how the rigid body will respond 
' when a collision has occurred with other objects in the simulation. See also neCollisionTable.
declare sub      RigidBodySetCollisionID(byval rb as neRigidBody ptr, byval cid as s32)
declare function RigidBodyGetCollisionID(byval rb as neRigidBody ptr) as s32

' Set and retrieve the linear damping factorof the rigid body.
declare sub      RigidBodySetLinearDamping(byval rb as neRigidBody ptr, byval damp as f32)
declare function RigidBodyGetLinearDamping(byval rb as neRigidBody ptr) as f32

' Set and retrieve the angular damping factor of the rigid body.
declare sub      RigidBodySetAngularDamping(byval rb as neRigidBody ptr, byval damp as f32)
declare function RigidBodyGetAngularDamping(byval rb as neRigidBody ptr) as f32

' Set and retrieve position of the rigid body.
declare sub      RigidBodySetPosition  (byval rb as neRigidBody ptr, byval x as f32, byval y as f32, byval z as f32)
declare sub      RigidBodyGetPosition  (byval rb as neRigidBody ptr, byref x as f32, byref y as f32, byref z as f32)
declare sub      RigidBodySetPositionV3(byval rb as neRigidBody ptr, byref p as const neV3)
declare function RigidBodyGetPositionV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve roation of the rigid body.
declare sub      RigidBodySetRotation(byval rb as neRigidBody ptr, byval p as f32, byval y as f32, byval r as f32)
declare sub      RigidBodyGetRotation(byval rb as neRigidBody ptr, byref p as f32, byref y as f32, byref r as f32)
declare sub      RigidBodySetRotationV3(byval rb as neRigidBody ptr, byref rotation as neV3)
declare function RigidBodyGetRotationV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve roation of the rigid body by quatrernion and matrix. 
declare sub      RigidBodySetRotationM3(byval rb as neRigidBody ptr, byref m as const neM3)
declare function RigidBodyGetRotationM3(byval rb as neRigidBody ptr) as neM3
declare sub      RigidBodySetRotationQ(byval rb as neRigidBody ptr, byref q as const neQ)
declare function RigidBodyGetRotationQ(byval rb as neRigidBody ptr) as neQ

' Set and retrieve position and roation of the rigid body.
declare sub      RigidBodySetPositionRotation  (byval rb as neRigidBody ptr, _
                                                byval px as f32, byval py as f32, byval pz as f32, _
                                                byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      RigidBodyGetPositionRotation  (byval rb as neRigidBody ptr, _
                                                byref px as f32, byref py as f32, byref pz as f32,  _
                                                byref rx as f32, byref ry as f32, byref rz as f32)
declare sub      RigidBodySetPositionRotationV3(byval rb as neRigidBody ptr, _
                                                byref position as const neV3, _
                                                byref rotation as const neV3)
declare sub      RigidBodyGetPositionRotationV3(byval rb as neRigidBody ptr, _
                                                byref position as neV3, _
                                                byref rotation as neV3)

declare sub      RigidBodySetTransform(byval rb as neRigidBody ptr, byref t as const neT3)
declare function RigidBodyGetTransform(byval rb as neRigidBody ptr) as neT3

' Set and retrieve dynamic states for the rigid body with these functions.
declare sub      RigidBodySetVelocity(byval rb as neRigidBody ptr, byref v as const neV3)
declare function RigidBodyGetVelocity(byval rb as neRigidBody ptr) as neV3

declare function RigidBodyGetVelocityAtPoint(byval rb as neRigidBody ptr, byref pt as const neV3) as neV3
declare function RigidBodyGetAngularVelocity(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetAngularMomentum(byval rb as neRigidBody ptr, byref am as const neV3)
declare function RigidBodyGetAngularMomentum(byval rb as neRigidBody ptr) as neV3

' Set and retrieve the force/torque acting on the rigid body.
' A force will accelerate the rigid body's velocity.
' A torque will accelerate the rigid body's angular velocity.
' When the force is applied at a point other than the Center of Mass of the rigid body, 
' it will also produce a torque, hence the second SetForce function, where position is defined in world coordinates.
' The force/torque will be apply continously. To apply no force/torque to the rigid body set the value to zero.
declare sub      RigidBodySetForceAtPos(byval rb as neRigidBody ptr, byref force as const neV3, byref position as const neV3)

declare sub      RigidBodySetForce(byval rb as neRigidBody ptr, byref force as const neV3)
declare function RigidBodyGetForce(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetTorque(byval rb as neRigidBody ptr, byref torque as const neV3)
declare function RigidBodyGetTorque(byval rb as neRigidBody ptr) as neV3

' Apply an impulse or a twist to the rigid body.
' An impulse will produce a immediate change in velocity.
' A twist will produce immediate a change in angular velocity.
' An impulse applied at a point other than the Center of Mass of the rigid body also produce a twist.
' To do this, use the second ApplyImpulse function, where position is define in world coordinates of the application point.
declare sub      RigidBodyApplyImpulse(byval rb as neRigidBody ptr, byref impulse as const neV3)
declare sub      RigidBodyApplyImpulseAtPos(byval rb as neRigidBody ptr, byref impulse as const neV3, byref position as const neV3)
declare sub      RigidBodyApplyTwist(byval rb as neRigidBody ptr, byref twist as const neV3)

' Enable or disable gravity for the rigid body.
' If gravity is enabled, the rigid body will have a force equal to gravity x mass
' automatically applied to it by the simulator.
declare sub      RigidBodyGravityEnable(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyIsGravityEnable(byval rb as neRigidBody ptr) as boolean

' Use these functions to create and retrieve neGeometry objects for the rigid body.
' The collection neGeometry assigned to a rigid body defines the body's collision boundary.
' It also defines the rigid body's breaking behaviour. 
' A rigid body can also exist and be simulated without any neGeomtry assigned to it; 
' it just won't collide with anything in the simulation. 
' All neGeometry objects within a rigid body will be freed automatcally when the rigid body is freed. See also neGeometry.
declare function RigidBodyAddGeometry(byval rb as neRigidBody ptr) as neGeometry ptr
declare function RigidBodyRemoveGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as boolean
declare function RigidBodyGetGeometryCount(byval rb as neRigidBody ptr) as s32
declare sub      RigidBodyBeginIterateGeometry(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextGeometry(byval rb as neRigidBody ptr) as neGeometry ptr

' Update the internal bounding information of the rigid body.
' Whenever the user changes the geometry and/or sensor of the rigid body 
' by adding, removing, or modifying the geometry and/or sensor, 
' this function needs to be called to update the internal bounding information.
declare sub      RigidBodyUpdateBoundingInfo(byval rb as neRigidBody ptr)

' Breaks off the geometry object, which must assigned to the rigid body. Returns the newly created rigid body.
declare function RigidBodyBreakGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as neRigidBody ptr

' Define whether or not a rigid body will collide with another body connected to it via joints (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable.
' (body which connected to this body indirectly)
declare sub      RigidBodySetCollideConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideConnected(byval rb as neRigidBody ptr)  as boolean
' (collide with any body which connected to this body directly)
declare sub      RigidBodySetCollideDirectlyConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideDirectlyConnected(byval rb as neRigidBody ptr) as boolean

declare sub      RigidBodySetUseCustomCollisionDetection(byval rb as neRigidBody ptr, byval yes as boolean, byval obb as const neT3 ptr, byval boundingRadius as f32)
declare function RigidBodyGetUseCustomCollisionDetection(byval rb as neRigidBody ptr) as boolean

' sensors of the rigid body. see neSensor
declare function RigidBodyAddSensor(byval rb as neRigidBody ptr) as neSensor ptr
declare function RigidBodyRemoveSensor(byval rb as neRigidBody ptr, byval sen as neSensor ptr) as boolean
declare sub      RigidBodyBeginIterateSensor(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextSensor(byval rb as neRigidBody ptr) as neSensor ptr

' activ state of the rigid body
declare sub      RigidBodySetSleepingParameter(byval rb as neRigidBody ptr, byval sleepingParam as f32)
declare function RigidBodyGetSleepingParameter(byval rb as neRigidBody ptr) as f32

' Activate or deactivate the rigid body.
' When a rigid body is deactivated, the simulator will not update its dynamic state, 
' will not call its controllers, and the rigid body will not collide with anything.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the rigid body being activated is.
' This can help speed up the introduction of the newly-activated rigid body into the simulation.
declare sub      RigidBodySetActive(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetActive(byval rb as neRigidBody ptr) as boolean
declare sub      RigidBodySetActiveWithRigidBodyHint   (byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      RigidBodySetActiveWithAnimatedBodyHint(byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

declare function RigidBodyIsIdle(byval rb as neRigidBody ptr) as boolean

declare sub      RigidBodyGetOpenGLMatrix(byval rb as neRigidBody ptr, byval m4x4 as f32 ptr)
declare sub      RigidBodyGetOpenGLMatrixInverted(byval rb as neRigidBody ptr, byval m4x4 as f32 ptr)
declare function RigidBodyRotateVector    (byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare function RigidBodyTransformVector (byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare sub      RigidBodyRotateVectors   (byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)
declare sub      RigidBodyTransformVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)

' #########################
' # neRigidBodyController #
' #########################
declare function RigidBodyAddController(byval rb as neRigidBody ptr, byval rbccb as neRigidBodyControllerCallback, byval period as s32=0) as neRigidBodyController ptr
declare function RigidBodyRemoveController(byval rb as neRigidBody ptr, byval controller as neRigidBodyController ptr) as boolean
declare sub      RigidBodyBeginIterateController(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextController(byval rb as neRigidBody ptr) as neRigidBodyController ptr

' use this only inside a rigidBoy controller callback
declare function RigidBodyControllerGetRigidBody(byval controller as neRigidBodyController ptr) as neRigidBody ptr

declare sub      RigidBodyControllerSetForceWithTorque(byval controller as neRigidBodyController ptr, _
                                                       byval fx as f32, byval fy as f32, byval fz as f32, _
                                                       byval px as f32, byval py as f32, byval pz as f32)

declare sub      RigidBodyControllerSetForce(byval controller as neRigidBodyController ptr, byval fx as f32, byval fy as f32, byval fz as f32)
declare sub      RigidBodyControllerGetForce(byval controller as neRigidBodyController ptr, byref fx as f32, byref fy as f32, byref fz as f32)

declare sub      RigidBodyControllerSetTorque(byval controller as neRigidBodyController ptr, byval tx as f32, byval ty as f32, byval tz as f32)
declare sub      RigidBodyControllerGetTorque(byval controller as neRigidBodyController ptr, byref tx as f32, byref ty as f32, byref tz as f32)

declare sub      RigidBodyControllerSetForceWithTorqueV3(byval controller as neRigidBodyController ptr, _
                                                         byref force as const neV3, byref position as const neV3)

declare sub      RigidBodyControllerSetForceV3(byval controller as neRigidBodyController ptr, byref force as const neV3)
declare function RigidBodyControllerGetForceV3(byval controller as neRigidBodyController ptr) as neV3

declare sub      RigidBodyControllerSetTorqueV3(byval controller as neRigidBodyController ptr, byref  torque as const neV3)
declare function RigidBodyControllerGetTorqueV3(byval controller as neRigidBodyController ptr) as neV3

' ############
' # neSensor #
' ############
' Specify and retrieve the user data associate with the neSensor object.
declare sub      SensorSetUserData          (byval sensor as neSensor ptr, byval userData as any ptr)
declare function SensorGetUserData          (byval sensor as neSensor ptr) as any ptr
' Specify the position and end vector of the line representing the sensor object.
declare sub      SensorSetLineSensor        (byval sensor as neSensor ptr, _
                                             byval px as f32, byval py as f32, byval pz as f32, _
                                             byval ex as f32, byval ey as f32, byval ez as f32)
declare sub      SensorSetLineSensorV3      (byval sensor as neSensor ptr, _
                                             byref position as const neV3, _
                                             byref lineVector as const neV3)
declare sub      SensorGetLineVector        (byval sensor as neSensor ptr, byref x as f32, byref y as f32, byref z as f32)
declare function SensorGetLineVectorV3      (byval sensor as neSensor ptr) as neV3
' Retrieve the current position and direction of the sensor.
declare sub      SensorGetLinePosition      (byval sensor as neSensor ptr, byref x as f32, byref y as f32, byref z as f32)
declare function SensorGetLinePositionV3    (byval sensor as neSensor ptr) as neV3

declare sub      SensorGetLineUnitVector    (byval sensor as neSensor ptr, byref x as f32, byref y as f32, byref z as f32)
declare function SensorGetLineUnitVectorV3  (byval sensor as neSensor ptr) as neV3

' NOTE: collision informations are only available inside a controller callback !

' If GetDetectDepth is <= 0.0, then the sensor does not intersect with other geometries
' and all other values returned by GetDetectXXX functions are invalid.
declare function SensorGetDetectDepth         (byval sensor as neSensor ptr) as f32

' Use these functions to retrieve the current intersection information of the sensor.
declare sub      SensorGetDetectNormal        (byval sensor as neSensor ptr, byref x as f32, byref y as f32, byref z as f32)
declare function SensorGetDetectNormalV3      (byval sensor as neSensor ptr) as neV3
declare sub      SensorGetDetectContactPoint  (byval sensor as neSensor ptr, byref x as f32, byref y as f32, byref z as f32)
declare function SensorGetDetectContactPointV3(byval sensor as neSensor ptr) as neV3
declare function SensorGetDetectRigidBody     (byval sensor as neSensor ptr) as neRigidBody ptr
declare function SensorGetDetectAnimatedBody  (byval sensor as neSensor ptr) as neAnimatedBody ptr
declare function SensorGetDetectMaterial      (byval sensor as neSensor ptr) as s32


' ###########
' # neJoint #
' ###########
' Specify and retrieve the user data associate with the neJoint object.
declare sub      JointSetUserData(byval joint as neJoint ptr, byval userData as any ptr)
declare function JointGetUserData(byval joint as neJoint ptr) as any ptr

' Specify and retrieve the type of the joint
type neConstraintType as u32
const as neConstraintType NE_JOINT_BALLSOCKET  = 0
const as neConstraintType NE_JOINT_HINGE       = 1
const as neConstraintType NE_JOINT_SLIDE       = 2
declare sub      JointSetType(byval joint as neJoint ptr, byval t as neConstraintType)
declare function JointGetType(byval joint as neJoint ptr) as neConstraintType

' Retrieve the bodies which are connected by this joint.
' The bodies are specified when the joint is created and cannot be changed.
declare function JointGetRigidBodyA   (byval joint as neJoint ptr) as neRigidBody ptr
declare function JointGetRigidBodyB   (byval joint as neJoint ptr) as neRigidBody ptr
declare function JointGetAnimatedBodyB(byval joint as neJoint ptr) as neAnimatedBody ptr

' Use these functions to specify and retrieve the reference frame of the joint.
' The reference frame of the joint specifies position and orientation 
' of the joint with respect to each body's local frame.

' For ball-and-socket joints, the position of the frame 
' specifies the location where the two bodies are connected.

' For hinge joints, the position specifies the location of the joint, 
' and the Y-axis of the joint frame is the axis of the joint 
' (which is the axis the two bodies are free to rotate about).
declare sub      JointSetFrameA(byval joint as neJoint ptr, byref frameA as const neT3)
declare function JointGetFrameA(byval joint as neJoint ptr) as neT3

declare sub      JointSetPositionRotationFrameA  (byval joint as neJoint ptr, _
                                                  byval px as f32, byval py as f32, byval pz as f32, _
                                                  byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      JointSetPositionRotationV3FrameA(byval joint as neJoint ptr, _
                                                  byref position as const neV3, _
                                                  byref rotation as const neV3)

declare sub      JointSetFrameB    (byval joint as neJoint ptr, byref frameB as const neT3)
declare function JointGetFrameB    (byval joint as neJoint ptr) as neT3

declare sub      JointSetPositionRotationFrameB  (byval joint as neJoint ptr, _
                                                  byval px as f32, byval py as f32, byval pz as f32, _
                                                  byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      JointSetPositionRotationV3FrameB(byval joint as neJoint ptr, _
                                                  byref position as const neV3, _
                                                  byref rotation as const neV3)

' Use this function to specify the joint's frame in world coordinate. 
' To use this function, typically the two bodies of the joint would already be positioned correctly 
' in relation to each other in world co-ordinates.
declare sub      JointSetFrameWorld(byval joint as neJoint ptr, byref frame as const neT3)
declare sub      JointSetPositionRotationFrameWorld(byval joint as neJoint ptr, _
                                                    byval px as f32, byval py as f32, byval pz as f32, _ 
                                                    byval rp as f32, byval ry as f32, byval rr as f32)
declare sub      JointSetPositionRotationV3FrameWorld(byval joint as neJoint ptr, _
                                                      byref position as const neV3, _
                                                      byref rotation as const neV3)

' Specify and retrieve the length of the joint.
' Some joints, such as hinge joints, have length as well as position and orientation.
declare sub      JointSetJointLength(byval joint as neJoint ptr, byval length as f32)
declare function JointGetJointLength(byval joint as neJoint ptr) as f32

' Enable or disable the joint. The second function returns true if the joint is enabled.
declare sub      JointEnable   (byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsEnabled(byval joint as neJoint ptr) as boolean

declare sub      JointSetDampingFactor(byval joint as neJoint ptr, byval factor as f32)
declare function JointGetDampingFactor(byval joint as neJoint ptr) as f32

' These functions relate to the accuracy of the solution of the joint.
' Generally, lower accuracy results in faster simulation, 
' but may be prone to side effects if the accuracy is too low.
' Epsilon specifies the early-exit theshold for the constraint solver.
' Iteration relates to the number of iterations the solver will use to solve the constraint.
' Setting the epsilon to 0 means the solver will always perform 
' the specified number of iterations and will not exit early.
declare sub      JointSetEpsilon(byval joint as neJoint ptr, byval epsilon as f32)
declare function JointGetEpsilon(byval joint as neJoint ptr) as f32

declare sub      JointSetIteration(byval joint as neJoint ptr, byval iteration as s32)
declare function JointGetIteration(byval joint as neJoint ptr) as s32

' Set the upper and lower limits of the joint.
' Joint limits are used to restrict the movement of a joint.

' For ball-and-socket joints and hinge joints, the joint's current angle 
' is the angle between the X-axes of the joint's frame in each of the joint bodies.

' For ball-and-socket joints, the valid range of the lower limit is 0 to PI; 
' for the upper limit, between lower the limit and PI. 

' For a hinge joint, the valid range for lower and upper limits is -PI..PI, 
' but the upper limit must be greater than the lower limit.

' For ball and socket joints, a twist limit can also be specified. 
' This is done by calling the SetUpperLimit2 and SetLowerLimit2 functions.
declare sub      JointSetUpperLimit(byval joint as neJoint ptr, byval limit as f32)
declare function JointGetUpperLimit(byval joint as neJoint ptr) as f32
declare sub      JointSetLowerLimit(byval joint as neJoint ptr, byval limit as f32)
declare function JointGetLowerLimit(byval joint as neJoint ptr) as f32
' joint limits 2
declare sub      JointSetUpperLimit2(byval joint as neJoint ptr, byval limit as f32)
declare function JointGetUpperLimit2(byval joint as neJoint ptr) as f32
declare sub      JointSetLowerLimit2(byval joint as neJoint ptr, byval limit as f32)
declare function JointGetLowerLimit2(byval joint as neJoint ptr) as f32

' Enable or disable the joint limit. 
' The second function returns true if the joint limit is enabled. 
' The last two functions relate to the second limit for the joint. 
' Note: currently, only ball and socket joints have a second limit, which is the twist. 
declare sub      JointLimitEnable    (byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsLimitEnabled (byval joint as neJoint ptr) as boolean
declare sub      JointLimit2Enable   (byval joint as neJoint ptr, byval yes as boolean) 
declare function JointIsLimit2Enabled(byval joint as neJoint ptr) as boolean

' joint motor
declare sub      JointMotorEnable(byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsMotorEnabled(byval joint as neJoint ptr) as boolean
declare sub      JointSetMotor(byval joint as neJoint ptr, byval desireValue as f32, byval maxForce as f32)
declare sub      JointGetMotor(byval joint as neJoint ptr, byref desireValue as f32, byref maxForce as f32)

' #####################
' # neJointController #
' #####################
' These functions provide access to adding and removing joint controller callbacks.
declare function JointAddController(byval joint as neJoint ptr, byval jccb as neJointControllerCallback, byval period as s32=0) as neJointController ptr
declare function JointRemoveController(byval joint as neJoint ptr, byval controller as neJointController ptr) as boolean

declare sub      JointBeginIterateController(byval joint as neJoint ptr)
declare function JointGetNextController(byval joint as neJoint ptr) as neJointController ptr

' use this only inside of a joint controller callback
declare function JointControllerGetJoint(byval controller as neJointController ptr) as neJoint ptr

declare sub      JointControllerSetForceV3BodyA(byval controller as neJointController ptr, byref  force as const neV3)
declare function JointControllerGetForceV3BodyA(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetForceV3BodyB(byval controller as neJointController ptr, byref  force as const neV3)
declare function JointControllerGetForceV3BodyB(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueV3BodyA(byval controller as neJointController ptr, byref  torque as const neV3)
declare function JointControllerGetTorqueV3BodyA(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueV3BodyB(byval controller as neJointController ptr, byref torque as const neV3)
declare function JointControllerGetTorqueV3BodyB(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetForceWithTorqueV3BodyA(byval controller as neJointController ptr, byref force as const neV3, byref pos as const neV3)
declare sub      JointControllerSetForceWithTorqueV3BodyB(byval controller as neJointController ptr, byref force as const neV3, byref pos as const neV3)

' ###########
' # helpers #
' ###########

' calculate the InertiaTensor for a Box
function neBoxInertiaTensor(byval fWidth  as f32, _
                            byval fHeight as f32, _
                            byval fDepth  as f32, _
                            byval fMass   as f32) as neV3
  dim as neV3 ret
  dim as f32 maxdim
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1   then fMass  =1
  maxdim = fWidth
  if (fHeight > maxdim) then maxdim = fHeight
  if (fDepth  > maxdim) then maxdim = fDepth
  dim as f32 xsq = maxdim
  dim as f32 ysq = maxdim
  dim as f32 zsq = maxdim

  fMass /= 3.0
  xsq *= xsq
  ysq *= ysq
  zsq *= zsq

  ret.x = (ysq + zsq) * fMass
  ret.y = (xsq + zsq) * fMass
  ret.z = (xsq + ysq) * fMass

  return ret
end function

' calculate the InertiaTensor for a Box
function neBoxInertiaTensorV3(byref vSize as const neV3, byval fMass as f32) as neV3
  return neBoxInertiaTensor(vSize.x, vSize.y, vSize.z, fMass)
end function

' calculate the InertiaTensor for a Cube
function neCubeInertiaTensor(byval fSize as f32, byval fMass as f32) as neV3
  return neBoxInertiaTensor(fSize, fSize, fSize, fMass)
end function

' calculate the InertiaTensor of a Sphere
function neSphereInertiaTensor(byval fDiameter as f32, byval fMass as f32) as neV3
  dim as neV3 ret
  dim as f32 radius
  dim as f32 value
  if fDiameter<0.1 then fDiameter=0.1
  if fMass    <1.0 then fMass    =1.0
  radius = fDiameter * 0.5
  value = 2.0 / 5.0 * fMass * radius * radius
  ret.x=value
  ret.y=value
  ret.z=value
  return ret
end function

' calculate the InertiaTensor of a Capsule
function neCapsuleInertiaTensor(byval fDiameter as f32, _
                                byval fHeight   as f32, _
                                byval fMass     as f32) as neV3
  dim as neV3 ret
  dim as f32 radius
  dim as f32 radiusSq
  dim as f32 Ixz
  dim as f32 Iyy

  if fDiameter<0.1 then fDiameter=0.1
  if fHeight  <0.1 then fHeight=0.1
  if fMass    <1    then fMass=1

  radius = fDiameter * 0.5
  radiusSq = radius * radius
  Ixz = 1.0 / 12.0 * fMass * fHeight * fHeight + 0.25 * fMass * radiusSq
  Iyy = 0.5 * fMass * radiusSq

  ret.x = Ixz
  ret.y = Iyy
  ret.z = Ixz
  return ret
end function

' create a static massless body with collision geometry
' !!! can only animated by the user !!!
function CreateAnimatedBodyBox(byval sim     as neSimulator ptr, _
                               byval fWidth  as f32=1, _
                               byval fHeight as f32=1, _
                               byval fDepth  as f32=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1 
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function

function CreateAnimatedBodyCube(byval sim   as neSimulator ptr, _
                                byval fSize as f32 = 1) as neAnimatedBody ptr
  return CreateAnimatedBodyBox(sim,fSize,fSize,fSize)
end function  

function CreateAnimatedBodySphere(byval sim       as neSimulator ptr, _
                                  byval fDiameter as f32=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter <0.1 then fDiameter=0.1
  GeometrySetSphereDiameter(geo, fDiameter)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function

function CreateAnimatedBodyCapsule(byval sim       as neSimulator ptr, _
                                   byval fDiameter as f32=0.5, _
                                   byval fHeight   as f32=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter<0.1 then fDiameter=0.1
  if fHeight  <0.1 then fHeight = 0.1
  GeometrySetCapsule(geo, fDiameter, fHeight)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function


' create a dynamic body with mass and collision geometry
function CreateRigidBodyBox(byval sim     as neSimulator ptr, _
                            byval fWidth  as f32=1, _
                            byval fHeight as f32=1, _
                            byval fDepth  as f32=1, _
                            byval fMass   as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1   then fMass  =1
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodySetInertiaTensor(rb,neBoxInertiaTensor(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidBodyCube(byval sim   as neSimulator ptr, _
                             byval fSize as f32=1, _
                             byval fMass as f32=1) as neRigidBody ptr
  return CreateRigidBodyBox(sim,fSize,fsize,fSize,fMass)
end function  


function CreateRigidBodySphere(byval sim       as neSimulator ptr, _
                               byval fDiameter as f32=1, _
                               byval fMass     as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter<0.1 then fDiameter =0.1
  if fMass  <1    then fMass  =1
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensor(rb,neSphereInertiaTensor(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidBodyCapsule(byval sim       as neSimulator ptr, _
                                byval fDiameter as f32=0.5, _
                                byval fHeight   as f32=1, _
                                byval fMass     as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.1 then fDiameter = 0.1
  if fHeight   < 0.1 then fHeight   = 0.1
  if fMass     < 1    then fMass     = 1
  GeometrySetCapsule(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensor(rb,neCapsuleInertiaTensor(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function


' Create a dynamic particle with mass and collision geometry
function CreateRigidParticleBox(byval sim     as neSimulator ptr, _
                                byval fWidth  as f32=1, _
                                byval fHeight as f32=1, _
                                byval fDepth  as f32=1, _
                                byval fMass   as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1   then fMass  =1
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodySetInertiaTensor(rb,neBoxInertiaTensor(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidParticleCube(byval sim   as neSimulator ptr, _
                                 byval fSize as f32=1, _
                                 byval fMass as f32=1) as neRigidBody ptr
  return CreateRigidParticleBox(sim,fSize,fSize,fSize,fMass)
end function


function CreateRigidParticleSphere(byval sim       as neSimulator ptr, _
                                   byval fDiameter as f32=1, _
                                   byval fMass     as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter<0.1 then fDiameter=0.1
  if fMass    <1   then fMass    =1
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensor(rb,neSphereInertiaTensor(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidParticleCapsule(byval sim       as neSimulator ptr, _
                                    byval fDiameter as f32=0.5, _
                                    byval fHeight   as f32=1, _
                                    byval fMass     as f32=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.1 then fDiameter = 0.1
  if fHeight   < 0.1 then fHeight   = 0.1
  if fMass     < 1   then fMass     = 1
  GeometrySetCapsule(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensor(rb,neCapsuleInertiaTensor(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function


' get pitch yaw roll from matrix
declare function RotAngle(byref m as const neM3) as neV3
' create matrix from pitch yaw roll vector
declare function RotMatrixV3(byref v as const neV3) as neM3
' create matrix from pitch yaw roll 
declare function RotMatrix(byval p as f32, byval y as f32, byval r as f32) as neM3

' get OpenGL matrix from frame
declare sub      FrameGetOpenGLMatrix(byref frame as const neT3, byval m4x4 as f32 ptr)
' get the inverse of the OpenGL matrix from frame
declare sub      FrameGetOpenGLMatrixInverted(byref frame as neT3, byval m4x4 as f32 ptr)

' use the frame to rotate a vector
declare function FrameRotateVector    (byref frame as const neT3, byref v as const neV3) as neV3
' use the frame to transform a vector
declare function FrameTransformVector (byref frame as neT3, byref v as const neV3) as neV3
' use the frame to rotate an array of vectors
declare sub      FrameRotateVectors   (byref frame as const neT3, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)
' use the frame to transform an array of vectors
declare sub      FrameTransformVectors(byref frame as neT3, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as s32)

' vector
declare sub      V3RemoveComponent(byref v as neV3, byref what as const neV3)
declare function V3Lerp(byref a as const neV3, byref b as const neV3, byval t as f32) as neV3

' rotation
declare sub      M3SetIdentiy(byref m as neM3)
declare function M3FromAxisAngle(byref axis as const neV3, byref angle as const f32) as neM3
declare function M3FromEulerAngle(byval p as f32, byval y as f32, byval r as f32) as neM3
declare function M3FromQ(byref q as const neQ) as neM3

declare function M3AddV3(byref m as const neM3, byref v as const neV3) as neV3
declare function M3SubV3(byref m as const neM3, byref v as const neV3) as neV3
declare function M3MulM3(byref a as const neM3, byref b as const neM3) as neM3
declare function M3Mul  (byref a as const neM3, byval b as f32) as neM3
declare function M3MulV3(byref m as const neM3, byref v as const neV3) as neV3

declare function M3Lerp(byref a as const neM3, byref b as const neM3, byval t as f32) as neM3

' rotation and translation
declare sub      T3SetIdentiy(byref t as neT3)
declare function T3MulV3(byref t as neT3, byref v as const neV3) as neV3
declare function T3MulT3(byref a as neT3, byref b as const neT3) as neT3
declare function T3Inverted(byref t as const neT3) as neT3
declare sub      T3Inverse(byref t as neT3)
' quaternion
declare sub      QNormalize(byref q as neQ)
declare function QNormalized(byref q as neQ) as neQ
declare function QFromM3(byref m as const neM3) as neQ
declare function QMulQ(byref a as const neQ, byref b as const neQ) as neQ
declare function QFromAxisAngle(byref axis as const neV3, byval angle as f32) as neQ
declare sub      QGetAxisAngle(byref q as const neQ, byref axis as neV3, byref angle as f32)
declare function QLerp(byref a as const neQ, byref b as const neQ,byval t as f32) as neQ

end extern



#endif ' __tokamak_c_bi__ 
